﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using System.IO;

//#if NET35
//using Microsoft.IdentityModel.Protocols.WSTrust;
//using Microsoft.IdentityModel.Tokens.Saml2;
//using Microsoft.IdentityModel.Tokens;
//using System.IdentityModel.Tokens;
//using Microsoft.IdentityModel.SecurityTokenService;
//#else
using System.IdentityModel.Tokens;
//using System.IdentityModel.Protocols.WSTrust;
//using System.Net;
//using System.IdentityModel.Configuration;
//using System.ServiceModel.Web;
using Microsoft.IdentityModel.SecurityTokenService;
using Microsoft.IdentityModel.Tokens;
using Microsoft.IdentityModel.Tokens.Saml2;
using WSTrust13RequestSerializer = Microsoft.IdentityModel.Protocols.WSTrust.WSTrust13RequestSerializer;
using WSTrust13ResponseSerializer = Microsoft.IdentityModel.Protocols.WSTrust.WSTrust13ResponseSerializer;
using WSTrustChannel = Microsoft.IdentityModel.Protocols.WSTrust.WSTrustChannel;
using WSTrustChannelFactory = Microsoft.IdentityModel.Protocols.WSTrust.WSTrustChannelFactory;
using WSTrustSerializationContext = Microsoft.IdentityModel.Protocols.WSTrust.WSTrustSerializationContext;
using Microsoft.IdentityModel.Protocols.WSTrust;

//#endif

namespace dk.nsi.seal
{
    class defaults
    {
        public static BasicHttpBinding BasicHttpBinding()
        {
            int max = 5000000;
            BasicHttpBinding defaultBinding = new BasicHttpBinding(BasicHttpSecurityMode.None)
            {
                MaxBufferSize = max,
                MaxReceivedMessageSize = max,
            };
            defaultBinding.ReaderQuotas.MaxArrayLength = max;
            return defaultBinding;
        }
    }
    
    public class Saml2SosiStsClient : IDisposable
    {
        public WSTrustChannelFactory ChannelFactory{get; private set;}
        public WSTrustChannel channel { get; private set; }

        public Saml2SosiStsClient(string configname)
        {
            ChannelFactory = new WSTrustChannelFactory(configname);
            ChannelFactory.TrustVersion = TrustVersion.WSTrust13;
            ChannelFactory.WSTrustResponseSerializer = new ResponseSerializer();
        }

        public Saml2SosiStsClient(EndpointAddress endpointAdr, Binding binding = null)
        {
            ChannelFactory = new WSTrustChannelFactory(binding??defaults.BasicHttpBinding(), endpointAdr)
            {
                TrustVersion = TrustVersion.WSTrust13,
                WSTrustResponseSerializer = new ResponseSerializer()
            };
        }
        
        public T ExchangeAssertion2<T>(Saml2Assertion assertion, Saml2Assertion healthAssertion, string appliesTo) where T : class
        {
            var rst = new RequestSecurityToken(RequestTypes.Issue)
            {
                Context = "urn:uuid:" + Guid.NewGuid().ToString("D"),
                ActAs = new SecurityTokenElement(new Saml2SecurityToken2(assertion, healthAssertion)),
//#if NET35
//                AppliesTo = new EndpointAddress(appliesTo),
//#else
                AppliesTo = new EndpointAddress(appliesTo),
//                AppliesTo = new EndpointReference(appliesTo),
//#endif
                TokenType = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0",
            };

            RequestSecurityTokenResponse rstr = null;
            var token = Channel.Issue(rst, out rstr) as SealSaml2SecurityToken;
            return SealUtilities.Deserialize<T>(token.assertion);
        }

        public WSTrustChannel Channel
        {
            get
            {
                if (channel == null)
                {
                    channel = (WSTrustChannel)ChannelFactory.CreateChannel();
                    channel.WSTrustSerializationContext.SecurityTokenHandlerCollectionManager.SecurityTokenHandlerCollections.First().Clear();
                    channel.WSTrustSerializationContext.SecurityTokenHandlerCollectionManager.SecurityTokenHandlerCollections.First().Add(new Saml2SecurityToken2TokenHandler());
                }
                return channel;
            }

            private set
            {
                if (channel != null)
                {
                    channel.Close();
                }
                channel = value;
            }
        }

        public void Dispose()
        {
            Channel = null;
            ChannelFactory.Close();
        }
    }

    public class Seal2SamlStsClient : IDisposable
    {
        public WSTrustChannelFactory ChannelFactory { get; private set; }
        public WSTrustChannel channel { get; private set; }

        public Seal2SamlStsClient (string configname)
        {
            ChannelFactory = new WSTrustChannelFactory(configname);
            ChannelFactory.TrustVersion = TrustVersion.WSTrust13;
        }

        public Seal2SamlStsClient(EndpointAddress endpointAdr, Binding binding = null)
        {
            ChannelFactory = new WSTrustChannelFactory(binding ?? defaults.BasicHttpBinding(), endpointAdr)
            {
                TrustVersion = TrustVersion.WSTrust13,
            };
        }

        public SecurityToken ExchangeAssertion<T>(T assertion, string appliesTo) where T : class
        {
            var xassertion = SealUtilities.Serialize<T>(assertion);

            var rst = new RequestSecurityToken(RequestTypes.Issue)
            {
                Context = "urn:uuid:" + Guid.NewGuid().ToString("D"),
                ActAs = new SecurityTokenElement(new SealSaml2SecurityToken(xassertion.Root)),
//#if NET35
//                AppliesTo = new EndpointAddress(appliesTo),
//#else
                //AppliesTo = new EndpointReference(appliesTo),
                AppliesTo = new EndpointAddress(appliesTo),
//#endif
                TokenType = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0",
            };
            RequestSecurityTokenResponse rstr = null;
            return Channel.Issue(rst, out rstr);
        }

        public WSTrustChannel Channel
        {
            get
            {
                if (channel == null)
                {
                    channel = (WSTrustChannel)ChannelFactory.CreateChannel();
                    channel.WSTrustSerializationContext.SecurityTokenHandlerCollectionManager.SecurityTokenHandlerCollections.First().Add(new SealSecurityTokenHandler());
                }
                return channel;
            }

            private set
            {
                if (channel != null)
                {
                    channel.Close();
                }
                channel = value;
            }
        }

        public void Dispose()
        {
            Channel = null;
            ChannelFactory.Close();
        }
    }

    class DGWSRequestSerializer : WSTrust13RequestSerializer
    {
        public override void WriteXmlElement(XmlWriter writer, string elementName, object elementValue, RequestSecurityToken rst, WSTrustSerializationContext context)
        {
            if (elementName != ns.DGWSAssertion)
            {
                if (elementName == "Claims")
                {
                    writer.WriteStartElement("Claims", ns.wst);
                    var xass = rst.Properties[ns.DGWSAssertion] as XDocument;
                    xass.Root.WriteTo(writer);
                    writer.WriteEndElement();
                }
                else
                {
                    base.WriteXmlElement(writer, elementName, elementValue, rst, context);
                }
            }
        }
    }
    
    class ResponseSerializer : WSTrust13ResponseSerializer
    {
        public override void ReadXmlElement(XmlReader reader, RequestSecurityTokenResponse rstr, WSTrustSerializationContext context)
        {
            if (reader.LocalName == "RequestedSecurityToken")
            {
                var rd = reader.ReadSubtree();
                rd.ReadToFollowing("Assertion", "urn:oasis:names:tc:SAML:2.0:assertion");
                var assdoc = XDocument.Load(rd.ReadSubtree());

                rstr.RequestedSecurityToken = new Microsoft.IdentityModel.Protocols.WSTrust.RequestedSecurityToken( new SealSaml2SecurityToken(assdoc.Root));
                rstr.Properties.Add(ns.DGWSAssertion, assdoc);
            }
            else
            {
                base.ReadXmlElement(reader, rstr, context);
            }
        }
    }


    class Saml2ResponseSerializer : WSTrust13ResponseSerializer
    {
        Saml2AssertionSerializer ser = new Saml2AssertionSerializer();

        public override void ReadXmlElement(XmlReader reader, RequestSecurityTokenResponse rstr, WSTrustSerializationContext context)
        {
            if (reader.LocalName == "RequestedSecurityToken")
            {
                var rd = reader.ReadSubtree();
                rd.ReadToFollowing("Assertion", "urn:oasis:names:tc:SAML:2.0:assertion");
                rstr.RequestedSecurityToken = new Microsoft.IdentityModel.Protocols.WSTrust.RequestedSecurityToken(new Saml2SecurityToken(ser.ReadSaml2Assertion(rd.ReadSubtree())));
            }
            else
            {
                base.ReadXmlElement(reader, rstr, context);
            }
        }
    }
}
