﻿using System;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.Xml;
using System.Xml.Serialization;
using Sundhedsjournalen.Service.Fmk;
using dk.nsi.seal;
using dk.nsi.seal.dgwstypes;

namespace Sundhedsjournalen.Service
{
    [ServiceContract]
    public interface IFmk
    {
        [OperationContract]
        string GetMedicineCard();
    }

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall)]
    public class FmkService : IFmk
    {
        public string GetMedicineCard()
        {
            var macert = new X509Certificate2(@"FmkCertifikater\Farooq Olsson.pkcs12", "Test1234");
            var sealcard = SealCard.Create(MakeFarooqOlssonAssertionForSts(macert));
            var sc = SealUtilities.SignIn(sealcard, "sundhed.dk", "http://pan.certifikat.dk/sts/services/SecurityTokenService", macert);

            var header = MakeHeader();

            var whitelistingheader = new WhitelistingHeader();
            whitelistingheader.OrgResponsibleName = "Sundhed.dk";
            whitelistingheader.OrgUsingID = new OrgUsingID { NameFormat = NameFormat.medcomskscode, Value = "16af2393" };
            whitelistingheader.RequestedRole = "Tandlæge";//"SOSU-assistent";
            whitelistingheader.SystemName = "Sundheddk";
            whitelistingheader.SystemOwnerName = "Sundhed.dk";
            whitelistingheader.SystemVersion = "1.0";

            MedicineCardType[] cardtypes;

            var client = new MedicineCardPortTypeClient();

            using (var scope = new OperationContextScope(client.InnerChannel))
            {
                OperationContext.Current.OutgoingMessageHeaders.Add(new SealCardMessageHeader(sc));
                PrescriptionReplicationStatusType pres;
                client.GetMedicineCard_2012_06_01(null, header, null, whitelistingheader, new GetMedicineCardRequestType
                {
                    PersonIdentifier = "0601010012",
                    IncludeNonRelevantPrescriptionMedications = false,
                },
                out pres, out cardtypes);
            }

            var sww = new StringWriter();
            var writer = XmlWriter.Create(sww);

            var x = new XmlSerializer(cardtypes.GetType());
            x.Serialize(writer, cardtypes);
            return sww.ToString();
        }

        static dk.nsi.seal.dgwstypes.Assertion MakeFarooqOlssonAssertionForSts(X509Certificate2 x509Certificate2)
        {
            var vnow = DateTimeEx.UtcNowRound - TimeSpan.FromMinutes(5);
            return new dk.nsi.seal.dgwstypes.Assertion
            {
                IssueInstant = vnow,
                id = "IDCard",
                Version = 2.0m,
                Issuer = "WinPLC",
                Conditions = new dk.nsi.seal.dgwstypes.Conditions
                {
                    NotBefore = vnow,
                    NotOnOrAfter = vnow + TimeSpan.FromHours(8)
                },
                Subject = new dk.nsi.seal.dgwstypes.Subject
                {
                    NameID = new dk.nsi.seal.dgwstypes.NameID
                    {
                        Format = SubjectIdentifierType.medcomcprnumber,// "http://rep.oio.dk/cpr.dk/xml/schemas/core/2005/03/18/CPR_PersonCivilRegistrationIdentifier.xsd",
                        Value = "1111141920"
                    },
                    SubjectConfirmation = new dk.nsi.seal.dgwstypes.SubjectConfirmation
                    {
                        ConfirmationMethod = dk.nsi.seal.dgwstypes.ConfirmationMethod.urnoasisnamestcSAML20cmholderofkey,
                        SubjectConfirmationData = new dk.nsi.seal.dgwstypes.SubjectConfirmationData
                        {
                            Item = new dk.nsi.seal.dgwstypes.KeyInfo
                            {
                                Item = "OCESSignature"
                            }
                        }
                    }
                },
                AttributeStatement = new[]
                {
                    new dk.nsi.seal.dgwstypes.AttributeStatement
                    {
                        id = dk.nsi.seal.dgwstypes.AttributeStatementID.IDCardData,
                        Attribute = new[] 
                        {
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.sosiIDCardID, AttributeValue = Guid.NewGuid().ToString("D")},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.sosiIDCardVersion, AttributeValue = "1.0.1"},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.sosiIDCardType, AttributeValue = "user"},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.sosiAuthenticationLevel, AttributeValue = "4"},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.sosiOCESCertHash, AttributeValue = x509Certificate2.GetCertHashString()}
                        }
                    },
                    new dk.nsi.seal.dgwstypes.AttributeStatement
                    {
                        id = dk.nsi.seal.dgwstypes.AttributeStatementID.UserLog,
                        Attribute = new[] 
                        {
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomUserCivilRegistrationNumber, AttributeValue = "1111141920" },
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomUserGivenName, AttributeValue = "Farooq"},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomUserSurName, AttributeValue = "Olsson"},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomUserEmailAddress, AttributeValue = "N/A"},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomUserRole, AttributeValue = "Tandlæge" },
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomUserAuthorizationCode, AttributeValue = "KSFDJ"}
                        }
                    },
                    new dk.nsi.seal.dgwstypes.AttributeStatement
                    {
                        id = dk.nsi.seal.dgwstypes.AttributeStatementID.SystemLog,
                        Attribute = new[] 
                        {
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomITSystemName, AttributeValue = "SOSITEST"},
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomCareProviderID, AttributeValue = "25520041", NameFormat = SubjectIdentifierType.medcomcvrnumber, NameFormatSpecified = true },
                            new dk.nsi.seal.dgwstypes.Attribute{ Name = AttributeName.medcomCareProviderName, AttributeValue = "HERLEV HOSPITAL TEST"},
                        }
                    }
                }
            };
        }

        static Fmk.Header MakeHeader()
        {
            return new Fmk.Header
            {
                SecurityLevel = 1,
                TimeOut = Fmk.TimeOut.Item1440,
                TimeOutSpecified = true,
                Linking = new Fmk.Linking
                {
                    FlowID = Guid.NewGuid().ToString("D"),
                    MessageID = Guid.NewGuid().ToString("D")
                },
                FlowStatus = Fmk.FlowStatus.flow_running,
                FlowStatusSpecified = true,
                Priority = Fmk.Priority.RUTINE,
                RequireNonRepudiationReceipt = Fmk.RequireNonRepudiationReceipt.yes
            };
        }
    }

    public class DateTimeEx
    {
        static public DateTime UtcNowRound
        {
            get
            {
                var n = DateTime.UtcNow;
                return new DateTime(n.Year, n.Month, n.Day, n.Hour, n.Minute, n.Second);
            }
        }
    }
}